# Generated by Django 4.2.4 on 2023-08-09 13:40
import uuid

import django.core.validators
import django.db.models.deletion
import django.utils.timezone
import parler.fields
import parler.models
import tinymce.models
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Product",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name="Created At"
                    ),
                ),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "seo_title",
                    models.CharField(
                        blank=True,
                        max_length=70,
                        null=True,
                        validators=[django.core.validators.MaxLengthValidator(70)],
                    ),
                ),
                (
                    "seo_description",
                    models.CharField(
                        blank=True,
                        max_length=300,
                        null=True,
                        validators=[django.core.validators.MaxLengthValidator(300)],
                    ),
                ),
                (
                    "seo_keywords",
                    models.CharField(
                        blank=True,
                        max_length=255,
                        null=True,
                        validators=[django.core.validators.MaxLengthValidator(255)],
                    ),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "product_code",
                    models.CharField(
                        default=uuid.uuid4,
                        max_length=100,
                        unique=True,
                        verbose_name="Product Code",
                    ),
                ),
                (
                    "slug",
                    models.SlugField(max_length=255, unique=True, verbose_name="Slug"),
                ),
                (
                    "price",
                    models.DecimalField(
                        decimal_places=2, max_digits=11, verbose_name="Price"
                    ),
                ),
                ("active", models.BooleanField(default=True, verbose_name="Active")),
                ("stock", models.PositiveIntegerField(default=0, verbose_name="Stock")),
                (
                    "discount_percent",
                    models.DecimalField(
                        decimal_places=2,
                        default=0.0,
                        max_digits=11,
                        verbose_name="Discount Percent",
                    ),
                ),
                ("hits", models.PositiveIntegerField(default=0, verbose_name="Hits")),
                (
                    "weight",
                    models.DecimalField(
                        decimal_places=2,
                        default=0.0,
                        max_digits=11,
                        verbose_name="Weight (kg)",
                    ),
                ),
                (
                    "final_price",
                    models.DecimalField(
                        decimal_places=2,
                        default=0.0,
                        editable=False,
                        max_digits=11,
                        verbose_name="Final Price",
                    ),
                ),
                (
                    "discount_value",
                    models.DecimalField(
                        decimal_places=2,
                        default=0.0,
                        editable=False,
                        max_digits=11,
                        verbose_name="Discount Value",
                    ),
                ),
                (
                    "price_save_percent",
                    models.DecimalField(
                        decimal_places=2,
                        default=0.0,
                        editable=False,
                        max_digits=11,
                        verbose_name="Price Save Percent",
                    ),
                ),
            ],
            options={
                "verbose_name": "Product",
                "verbose_name_plural": "Products",
                "ordering": ["-created_at"],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="ProductCategory",
            fields=[
                (
                    "sort_order",
                    models.IntegerField(
                        db_index=True,
                        editable=False,
                        null=True,
                        verbose_name="Sort Order",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name="Created At"
                    ),
                ),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "seo_title",
                    models.CharField(
                        blank=True,
                        max_length=70,
                        null=True,
                        validators=[django.core.validators.MaxLengthValidator(70)],
                    ),
                ),
                (
                    "seo_description",
                    models.CharField(
                        blank=True,
                        max_length=300,
                        null=True,
                        validators=[django.core.validators.MaxLengthValidator(300)],
                    ),
                ),
                (
                    "seo_keywords",
                    models.CharField(
                        blank=True,
                        max_length=255,
                        null=True,
                        validators=[django.core.validators.MaxLengthValidator(255)],
                    ),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "slug",
                    models.SlugField(max_length=255, unique=True, verbose_name="Slug"),
                ),
                (
                    "menu_image_one",
                    models.ImageField(
                        blank=True,
                        null=True,
                        upload_to="uploads/categories/",
                        verbose_name="Menu Image One",
                    ),
                ),
                (
                    "menu_image_two",
                    models.ImageField(
                        blank=True,
                        null=True,
                        upload_to="uploads/categories/",
                        verbose_name="Menu Image Two",
                    ),
                ),
                (
                    "menu_main_banner",
                    models.ImageField(
                        blank=True,
                        null=True,
                        upload_to="uploads/categories/",
                        verbose_name="Menu Main Banner",
                    ),
                ),
                ("lft", models.PositiveIntegerField(editable=False)),
                ("rght", models.PositiveIntegerField(editable=False)),
                ("tree_id", models.PositiveIntegerField(db_index=True, editable=False)),
                ("level", models.PositiveIntegerField(editable=False)),
            ],
            options={
                "verbose_name": "Product Category",
                "verbose_name_plural": "Product Categories",
                "ordering": ["sort_order"],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="ProductCategoryTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        blank=True, max_length=255, null=True, verbose_name="Name"
                    ),
                ),
                (
                    "description",
                    tinymce.models.HTMLField(
                        blank=True, null=True, verbose_name="Description"
                    ),
                ),
            ],
            options={
                "verbose_name": "Product Category Translation",
                "db_table": "product_productcategory_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="ProductFavourite",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name="Created At"
                    ),
                ),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
            ],
            options={
                "verbose_name": "Product Favourite",
                "verbose_name_plural": "Product Favourites",
                "ordering": ["-updated_at"],
            },
        ),
        migrations.CreateModel(
            name="ProductImage",
            fields=[
                (
                    "sort_order",
                    models.IntegerField(
                        db_index=True,
                        editable=False,
                        null=True,
                        verbose_name="Sort Order",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name="Created At"
                    ),
                ),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "image",
                    models.ImageField(
                        upload_to="uploads/products/", verbose_name="Image"
                    ),
                ),
                (
                    "thumbnail",
                    models.ImageField(
                        blank=True,
                        null=True,
                        upload_to="uploads/products/thumbnails/",
                        verbose_name="Thumbnail",
                    ),
                ),
                ("is_main", models.BooleanField(default=False, verbose_name="Is Main")),
            ],
            options={
                "verbose_name": "Product Image",
                "verbose_name_plural": "Product Images",
                "ordering": ["sort_order"],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="ProductImageTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        blank=True, max_length=50, null=True, verbose_name="Title"
                    ),
                ),
            ],
            options={
                "verbose_name": "Product Image Translation",
                "db_table": "product_productimage_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="ProductReview",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name="Created At"
                    ),
                ),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "published_at",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="Published At"
                    ),
                ),
                (
                    "is_published",
                    models.BooleanField(default=False, verbose_name="Is Published"),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "rate",
                    models.PositiveSmallIntegerField(
                        choices=[
                            (1, "ONE"),
                            (2, "TWO"),
                            (3, "THREE"),
                            (4, "FOUR"),
                            (5, "FIVE"),
                            (6, "SIX"),
                            (7, "SEVEN"),
                            (8, "EIGHT"),
                            (9, "NINE"),
                            (10, "TEN"),
                        ],
                        verbose_name="Rate",
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[("New", "NEW"), ("True", "TRUE"), ("False", "FALSE")],
                        default="New",
                        max_length=250,
                        verbose_name="Status",
                    ),
                ),
                (
                    "product",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="product_review_product",
                        to="product.product",
                    ),
                ),
            ],
            options={
                "verbose_name": "Product Review",
                "verbose_name_plural": "Product Reviews",
                "ordering": ["-created_at"],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="ProductTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        blank=True, max_length=255, null=True, verbose_name="Name"
                    ),
                ),
                (
                    "description",
                    tinymce.models.HTMLField(
                        blank=True, null=True, verbose_name="Description"
                    ),
                ),
                (
                    "master",
                    parler.fields.TranslationsForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="product.product",
                    ),
                ),
            ],
            options={
                "verbose_name": "Product Translation",
                "db_table": "product_product_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="ProductReviewTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "comment",
                    models.CharField(
                        blank=True, max_length=250, null=True, verbose_name="Comment"
                    ),
                ),
                (
                    "master",
                    parler.fields.TranslationsForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="product.productreview",
                    ),
                ),
            ],
            options={
                "verbose_name": "Product Review Translation",
                "db_table": "product_productreview_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
    ]
